@@ -47,6 +47,11 @@ void dGeomMoved (dxGeom *geom)
 {
   dAASSERT (geom);
   
+  // if geom is offset, mark it as needing a calculate
+  if (geom->offset_posr) {
+    geom->gflags |= GEOM_POSR_BAD;
+  }
+  
   // from the bottom of the space heirarchy up, process all clean geoms
   // turning them into dirty geoms.
   dxSpace *parent = geom->parent_space;
