@@ -33,7 +33,7 @@
 #define dsDrawBox dsDrawBoxD
 #define dsDrawSphere dsDrawSphereD
 #define dsDrawCylinder dsDrawCylinderD
-#define dsDrawCappedCylinder dsDrawCappedCylinderD
+#define dsDrawCapsule dsDrawCapsuleD
 #define dsDrawLine dsDrawLineD
 #define dsDrawTriangle dsDrawTriangleD
 #endif
@@ -52,6 +52,10 @@
 struct MyObject {
   dBodyID body;			// the body
   dGeomID geom[GPB];		// geometries representing this body
+
+  // Trimesh only - double buffered matrices for 'last transform' setup
+  dReal matrix_dblbuff[ 16 * 2 ];
+  int last_matrix_index;
 };
 
 static int num=0;		// number of objects in simulation
@@ -1483,9 +1487,10 @@ static void start()
   printf ("To drop another object, press:\n");
   printf ("   b for box.\n");
   printf ("   s for sphere.\n");
-  printf ("   c for cylinder.\n");
+  printf ("   y for cylinder.\n");
+  printf ("   c for capsule.\n");
   printf ("   x for a composite object.\n");
-  printf ("   m for a trimesh (EXPERIMENTAL).\n");
+  printf ("   m for a trimesh.\n");
   printf ("To select an object, press space.\n");
   printf ("To disable the selected object, press d.\n");
   printf ("To enable the selected object, press e.\n");
@@ -1511,8 +1516,7 @@ static void command (int cmd)
   dMass m;
 
   cmd = locase (cmd);
-  if (cmd == 'b' || cmd == 's' || cmd == 'c' || cmd == 'x' || cmd == 'm'
-      /* || cmd == 'l' */) {
+  if (cmd == 'b' || cmd == 's' || cmd == 'c' || cmd == 'x' || cmd == 'm' || cmd == 'y' ) {
     if (num < NUM) {
       i = num;
       num++;
@@ -1550,7 +1554,7 @@ static void command (int cmd)
       dRFromAxisAndAngle (R,0,0,1,dRandReal()*10.0-5.0);
     }
     dBodySetRotation (obj[i].body,R);
-    dBodySetData (obj[i].body,(void*) i);
+    dBodySetData (obj[i].body,(void*)(size_t)i);
 
     if (cmd == 'b') {
       dMassSetBox (&m,DENSITY,sides[0],sides[1],sides[2]);
@@ -1558,18 +1562,15 @@ static void command (int cmd)
     }
     else if (cmd == 'c') {
       sides[0] *= 0.5;
-      dMassSetCappedCylinder (&m,DENSITY,3,sides[0],sides[1]);
-      obj[i].geom[0] = dCreateCCylinder (space,sides[0],sides[1]);
+      dMassSetCapsule (&m,DENSITY,3,sides[0],sides[1]);
+      obj[i].geom[0] = dCreateCapsule (space,sides[0],sides[1]);
     }
-/*
-    // cylinder option not yet implemented
-    else if (cmd == 'l') {
+    else if (cmd == 'y') {
       sides[1] *= 0.5;
-      dMassSetCappedCylinder (&m,DENSITY,3,sides[0],sides[1]);
+      dMassSetCylinder (&m,DENSITY,3,sides[0],sides[1]);
       obj[i].geom[0] = dCreateCylinder (space,sides[0],sides[1]);
     }
-*/
-    else if (cmd == 's') {
+	else if (cmd == 's') {
       sides[0] *= 0.5;
       dMassSetSphere (&m,DENSITY,sides[0]);
       obj[i].geom[0] = dCreateSphere (space,sides[0]);
@@ -1581,9 +1582,10 @@ static void command (int cmd)
       obj[i].geom[0] = dCreateTriMesh(space, new_tmdata, 0, 0, 0);
 
       // remember the mesh's dTriMeshDataID on its userdata for convenience.
-      dGeomSetData(obj[i].geom[0], new_tmdata);      
+      dGeomSetData(obj[i].geom[0], new_tmdata);
 
-      dMassSetBox (&m,DENSITY,sides[0],sides[1],sides[2]);
+	  dMassSetTrimesh( &m, DENSITY, obj[i].geom[0] );
+//    dMassSetBox (&m,DENSITY,sides[0],sides[1],sides[2]);
     }
     else if (cmd == 'x') {
       dGeomID g2[GPB];		// encapsulated geometries
@@ -1613,8 +1615,8 @@ static void command (int cmd)
 	else {
 	  dReal radius = dRandReal()*0.1+0.05;
 	  dReal length = dRandReal()*1.0+0.1;
-	  g2[k] = dCreateCCylinder (0,radius,length);
-	  dMassSetCappedCylinder (&m2,DENSITY,3,radius,length);
+	  g2[k] = dCreateCapsule (0,radius,length);
+	  dMassSetCapsule (&m2,DENSITY,3,radius,length);
 	}
 	dGeomTransformSetGeom (obj[i].geom[k],g2[k]);
 
@@ -1688,19 +1690,17 @@ void drawGeom (dGeomID g, const dReal *pos, const dReal *R, int show_aabb)
   else if (type == dSphereClass) {
     dsDrawSphere (pos,R,dGeomSphereGetRadius (g));
   }
-  else if (type == dCCylinderClass) {
+  else if (type == dCapsuleClass) {
     dReal radius,length;
-    dGeomCCylinderGetParams (g,&radius,&length);
-    dsDrawCappedCylinder (pos,R,length,radius);
+    dGeomCapsuleGetParams (g,&radius,&length);
+    dsDrawCapsule (pos,R,length,radius);
   }
-/*
-  // cylinder option not yet implemented
   else if (type == dCylinderClass) {
     dReal radius,length;
     dGeomCylinderGetParams (g,&radius,&length);
     dsDrawCylinder (pos,R,length,radius);
   }
-*/
+
   else if (type == dGeomTransformClass) {
     dGeomID g2 = dGeomTransformGetGeom (g);
     const dReal *pos2 = dGeomGetPosition (g2);
@@ -1731,12 +1731,48 @@ void drawGeom (dGeomID g, const dReal *pos, const dReal *R, int show_aabb)
 }
 
 
+// set previous transformation matrix for trimesh
+void setCurrentTransform(dGeomID geom)
+{
+ const dReal* Pos = dGeomGetPosition(geom);
+ const dReal* Rot = dGeomGetRotation(geom);
+
+ const dReal Transform[16] = 
+ {
+   Rot[0], Rot[4], Rot[8],  0,
+   Rot[1], Rot[5], Rot[9],  0,
+   Rot[2], Rot[6], Rot[10], 0,
+   Pos[0], Pos[1], Pos[2],  1
+ };
+
+ dGeomTriMeshSetLastTransform( geom, *(dMatrix4*)(&Transform) );
+
+}
+
+
 // simulation loop
 
 static void simLoop (int pause)
 {
   dsSetColor (0,0,2);
   dSpaceCollide (space,0,&nearCallback);
+
+
+#if 0
+  // What is this for??? - Bram
+  if (!pause) 
+  {
+    for (int i=0; i<num; i++)
+      for (int j=0; j < GPB; j++)
+        if (obj[i].geom[j])
+          if (dGeomGetClass(obj[i].geom[j]) == dTriMeshClass)
+            setCurrentTransform(obj[i].geom[j]);
+ 
+    setCurrentTransform(TriMesh1);
+    setCurrentTransform(TriMesh2);
+  }
+#endif
+
   //if (!pause) dWorldStep (world,0.05);
   if (!pause) dWorldStepFast1 (world,0.05, 5);
 
@@ -1786,15 +1822,21 @@ static void simLoop (int pause)
 
           // tell the tri-tri collider the current transform of the trimesh --
           // this is fairly important for good results.
-          /*
-            // NOT IMPLEMENTED YET
-          dTriMeshDataID TriMeshData = geomGetData(obj[i].geom[j]);
-          const double *DoubleArrayPtr =  Bodies[BodyIndex].TransformationMatrix->GetArray();
-          dGeomTriMeshDataSet( TriMeshData,
-                               TRIMESH_LAST_TRANSFORMATION,
-                               (void *) DoubleArrayPtr );
-          */
+          
+		  // Fill in the (4x4) matrix.
+		  dReal* p_matrix = obj[i].matrix_dblbuff + ( obj[i].last_matrix_index * 16 );
 
+		  p_matrix[ 0 ] = Rot[ 0 ];	p_matrix[ 1 ] = Rot[ 1 ];	p_matrix[ 2 ] = Rot[ 2 ];	p_matrix[ 3 ] = 0;
+		  p_matrix[ 4 ] = Rot[ 4 ];	p_matrix[ 5 ] = Rot[ 5 ];	p_matrix[ 6 ] = Rot[ 6 ];	p_matrix[ 7 ] = 0;
+		  p_matrix[ 8 ] = Rot[ 8 ];	p_matrix[ 9 ] = Rot[ 9 ];	p_matrix[10 ] = Rot[10 ];	p_matrix[11 ] = 0;
+		  p_matrix[12 ] = Pos[ 0 ];	p_matrix[13 ] = Pos[ 1 ];	p_matrix[14 ] = Pos[ 2 ];	p_matrix[15 ] = 1;
+
+		  // Flip to other matrix.
+		  obj[i].last_matrix_index = !obj[i].last_matrix_index;
+
+		  dGeomTriMeshSetLastTransform( obj[i].geom[j], 
+			  *(dMatrix4*)( obj[i].matrix_dblbuff + obj[i].last_matrix_index * 16 ) );
+  
         } else {
           drawGeom (obj[i].geom[j],0,0,show_aabb);
         }
@@ -1852,9 +1894,13 @@ int main (int argc, char **argv)
   fn.command = &command;
   fn.stop = 0;
   fn.path_to_textures = "../../drawstuff/textures";
+  if(argc==2)
+    {
+        fn.path_to_textures = argv[1];
+    }
 
   // create world
-
+  dInitODE();
   world = dWorldCreate();
  
   space = dSimpleSpaceCreate(0);
@@ -1872,6 +1918,8 @@ int main (int argc, char **argv)
   
   TriMesh1 = dCreateTriMesh(space, TriData1, 0, 0, 0);
   TriMesh2 = dCreateTriMesh(space, TriData2, 0, 0, 0);
+  dGeomSetData(TriMesh1, TriData1);
+  dGeomSetData(TriMesh2, TriData2);
   
   {dGeomSetPosition(TriMesh1, 0, 0, 0.9);
   dMatrix3 Rotation;
@@ -1889,6 +1937,6 @@ int main (int argc, char **argv)
   dJointGroupDestroy (contactgroup);
   dSpaceDestroy (space);
   dWorldDestroy (world);
-
+  dCloseODE();
   return 0;
 }
